@@ -24,6 +24,22 @@ gettext.install("sticky", "/usr/share/locale", names="ngettext")
 APPLICATION_ID = 'org.x.sticky'
 STYLE_SHEET_PATH = '/usr/share/sticky/sticky.css'
 SCHEMA = 'org.x.sticky'
+DBUS_PATH = '/org/x/sticky'
+DBUS_INTERFACE_XML = '''
+<node>
+  <interface name='org.x.sticky'>
+    <method name='ShowNotes'>
+    </method>
+    <method name='NewNote'>
+      <arg type='s' name='text' direction='in'/>
+    </method>
+    <method name='NewNoteBlank'>
+    </method>
+    <signal name='NotesChanged'>
+    </signal>
+  </interface>
+</node>
+'''
 
 UPDATE_DELAY = 1
 
@@ -76,6 +92,18 @@ SHORTCUTS = {
     ]
 }
 
+START_POSITIONS = {
+    'top-left': _("Top Left"),
+    'top-center': _("Top Center"),
+    'top-right': _("Top Right"),
+    'center-left': _("Center Left"),
+    'center-center': _("Center"),
+    'center-right': _("Center Right"),
+    'bottom-left': _("Bottom Left"),
+    'bottom-center': _("Bottom Center"),
+    'bottom-right': _("Bottom Right"),
+}
+
 class Note(Gtk.Window):
     @GObject.Signal(flags=GObject.SignalFlags.RUN_LAST, return_type=bool,
                     accumulator=GObject.signal_accumulator_true_handled)
@@ -586,6 +614,8 @@ class SettingsWindow(XApp.PreferencesWindow):
         page = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         page.pack_start(GSettingsSpinButton(_("Default height"), SCHEMA, 'default-height', mini=50, maxi=2000, step=10), False, False, 0)
         page.pack_start(GSettingsSpinButton(_("Default width"), SCHEMA, 'default-width', mini=50, maxi=2000, step=10), False, False, 0)
+        start_posititions = [(x, y) for x, y in START_POSITIONS.items()]
+        page.pack_start(GSettingsComboBox(_("Default position"), SCHEMA, 'default-position', options=start_posititions, valtype=str), False, False, 0)
         try:
             colors = [(x, y) for x, y in COLORS.items()]
             colors.append(('sep', ''))
@@ -646,8 +676,8 @@ class ShortcutsWindow(Gtk.ShortcutsWindow):
 
 class Application(Gtk.Application):
 
-    def __init__(self):
-        super(Application, self).__init__(application_id=APPLICATION_ID, flags=Gio.ApplicationFlags.FLAGS_NONE)
+    def __init__(self, *args):
+        super(Application, self).__init__(*args, application_id=APPLICATION_ID, flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
 
         self.status_icon = None
         self.has_activated = False
@@ -660,6 +690,55 @@ class Application(Gtk.Application):
         self.manager = None
         self.notes_hidden = False
         self.autostart_mode = False # indicates if we're in autostart mode
+        self.dbus_register_id = 0
+
+        self.add_main_option(
+            'autostart',
+            ord('a'),
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.NONE,
+            'Autostart mode',
+            None)
+        self.add_main_option(
+            'new',
+            ord('n'),
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.NONE,
+            'Create a new note',
+            None)
+
+        self.add_main_option(
+            'toggle',
+            ord('t'),
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.NONE,
+            'Toggle note visibility',
+            None)
+
+    def do_command_line(self, command_line):
+        options = command_line.get_options_dict()
+        options = options.end().unpack()
+
+        if 'autostart' in options:
+            settings = Gio.Settings(schema_id=SCHEMA)
+            if not settings.get_boolean("autostart"):
+                sys.exit()
+            else:
+                self.autostart_mode = True
+
+        if not self.has_activated:
+            self.activate()
+
+        if 'new' in options:
+            self.new_note(None)
+
+        if 'toggle' in options:
+            if self.notes_hidden:
+                self.activate_notes(Gtk.get_current_event_time())
+            else:
+                self.hide_notes()
+
+        return 0
 
     def do_activate(self):
         if self.has_activated:
@@ -688,6 +767,7 @@ class Application(Gtk.Application):
         self.file_handler.connect('lists-changed', self.on_lists_changed)
         self.group_update_id = self.file_handler.connect('group-changed', self.on_group_changed)
         self.file_handler.connect('group-name-changed', self.on_group_name_changed)
+        self.file_handler.connect('saved', self.on_save)
 
         if self.settings.get_boolean('show-in-tray'):
             self.create_status_icon()
@@ -728,6 +808,33 @@ class Application(Gtk.Application):
 
         self.has_activated = True
 
+    def do_dbus_register(self, connection, object_path):
+        dbus_interface_info = Gio.DBusNodeInfo.new_for_xml(DBUS_INTERFACE_XML)
+        self.dbus_register_id = connection.register_object(
+            object_path, dbus_interface_info.interfaces[0], self.dbus_method_callback, None, None)
+
+        return Gio.Application.do_dbus_register(self, connection, object_path)
+
+    def do_dbus_unregister(self, connection, object_path):
+        if self.dbus_register_id > 0:
+            connection.unregister_object(self.dbus_register_id)
+            self.dbus_register_id = 0
+
+
+        Gio.Application.do_dbus_unregister(self, connection, object_path)
+
+    def dbus_method_callback(self, connection, sender, path, iface_name, method_name, params, invocation, user_data=None):
+        if method_name == 'ShowNotes':
+            self.activate_notes(0)
+
+        elif method_name == 'NewNote':
+            x, y = self.find_note_location(40, 40)
+
+            self.add_note({'text': params.unpack()[0], 'x': x, 'y': y})
+
+        elif method_name == 'NewNoteBlank':
+            self.new_note()
+
     def first_run(self):
         gnote_dir = os.path.join(GLib.get_user_data_dir(), 'gnote')
 
@@ -874,23 +981,57 @@ class Application(Gtk.Application):
         self.notes_hidden = True
         self.update_dummy_window()
 
-    def new_note(self, button=None, parent=None):
-        if parent is None:
-            x = 40
-            y = 40
-        else:
-            x = parent.x + 20
-            y = parent.y + 20
+    def find_note_location(self, x, y, direction):
         while True:
             found = False
             for note_info in self.file_handler.get_note_list(self.note_group):
                 if note_info['x'] == x and note_info['y'] == y:
                     found = True
                     break
+
             if not found:
                 break
-            x += 20
-            y += 20
+
+            x += 20 * direction[0]
+            y += 60 * direction[1]
+
+        return x, y
+
+    def new_note(self, button=None, parent=None):
+        workarea_rect = Gdk.Monitor.get_workarea(
+            Gdk.Display.get_primary_monitor(Gdk.Display.get_default())
+        )
+        direction = [1, 1]
+        if parent:
+            if parent.x > (workarea_rect.width / 2):
+                x = parent.x - 20
+                direction[0] = -1
+            else:
+                x = parent.x + 20
+            if parent.y > (workarea_rect.height / 2):
+                y = parent.y - 60
+                direction[1] = -1
+            else:
+                y = parent.y + 60
+        else:
+            pos_y, pos_x = self.settings.get_string('default-position').split('-')
+            if pos_x == 'right':
+                x = workarea_rect.width - self.settings.get_uint('default-width') - 20
+                direction[0] = -1
+            elif pos_x == 'center':
+                x = workarea_rect.width / 2 - self.settings.get_uint('default-width') / 2
+            else:
+                x = 20
+            if pos_y == 'bottom':
+                y = workarea_rect.height - self.settings.get_uint('default-height') - 60
+                direction[1] = -1
+            elif pos_y == 'center':
+                y = workarea_rect.height / 2 - self.settings.get_uint('default-height') / 2
+            else:
+                y = 20
+
+        x, y = self.find_note_location(x, y, direction)
+
         info = {'x': x, 'y': y}
 
         self.add_note(info)
@@ -954,6 +1095,9 @@ class Application(Gtk.Application):
         if self.settings.get_string('active-group') == old_name:
             self.settings.set_string('active-group', new_name)
 
+    def on_save(self, *args):
+        self.get_dbus_connection().emit_signal(None, DBUS_PATH, APPLICATION_ID, 'NotesChanged', None)
+
     def change_visible_note_group(self, group=None):
         default = self.settings.get_string('active-group')
         if group is None:
@@ -1013,7 +1157,7 @@ class Application(Gtk.Application):
         dlg.set_program_name(_("Notes"))
         dlg.set_comments(_("Take notes and stay organized"))
         try:
-            h = open('/usr/share/licenses/common/GPL/license.txt', encoding="utf-8")
+            h = open('/usr/share/licenses/spdx/GPL-2.0-only.txt', encoding="utf-8")
             s = h.readlines()
             gpl = ""
             for line in s:
@@ -1023,7 +1167,7 @@ class Application(Gtk.Application):
         except Exception as e:
             print (e)
 
-        dlg.set_version("1.16")
+        dlg.set_version("1.24")
         dlg.set_icon_name("sticky")
         dlg.set_logo_icon_name("sticky")
         dlg.set_website("https://www.github.com/linuxmint/sticky")
@@ -1070,14 +1214,5 @@ class Application(Gtk.Application):
         self.quit()
 
 if __name__ == "__main__":
-    autostart_mode = False
-    if "--autostart" in sys.argv:
-        settings = Gio.Settings(schema_id=SCHEMA)
-        if not settings.get_boolean("autostart"):
-            sys.exit()
-        else:
-            autostart_mode = True
-
     sticky = Application()
-    sticky.autostart_mode = autostart_mode
-    sticky.run()
+    sticky.run(sys.argv)
